@@ -19,7 +19,7 @@
 */
 
 /*
- * $Id: commands.c 6311 2012-05-26 15:33:07Z rousseau $
+ * $Id: commands.c 6570 2013-03-14 09:12:47Z rousseau $
  */
 
 #include <string.h>
@@ -855,6 +855,8 @@ again:
 		goto end;
 	}
 
+time_request:
+	length_out = 10 + *RxLength;
 	res = ReadPort(reader_index, &length_out, cmd_out);
 
 	/* replay the command if NAK
@@ -885,6 +887,12 @@ again:
 		goto end;
 	}
 
+	if (cmd_out[STATUS_OFFSET] & CCID_TIME_EXTENSION)
+	{
+		DEBUG_COMM2("Time extension requested: 0x%02X", cmd_out[ERROR_OFFSET]);
+		goto time_request;
+	}
+
 	if (cmd_out[STATUS_OFFSET] & CCID_COMMAND_FAILED)
 	{
 		ccid_error(cmd_out[ERROR_OFFSET], __FILE__, __LINE__, __FUNCTION__);    /* bError */
@@ -1701,14 +1709,15 @@ static RESPONSECODE T0ProcACK(unsigned int reader_index,
 	unsigned int proc_len, int is_rcv)
 {
 	RESPONSECODE return_value;
-	unsigned int remain_len;
-	unsigned char tmp_buf[512];
 	unsigned int ret_len;
 
 	DEBUG_COMM2("Enter, is_rcv = %d", is_rcv);
 
 	if (is_rcv == 1)
 	{	/* Receiving mode */
+		unsigned int remain_len;
+		unsigned char tmp_buf[512];
+
 		if (*in_len > 0)
 		{	/* There are still available data in our buffer */
 			if (*in_len >= proc_len)
